// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/*-
 * Copyright (c) 2015 Adam Bieńkowski (https://launchpad.net/switchboard-plug-parental-controls)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authored by: Adam Bieńkowski <donadigos159@gmail.com>
 */

namespace PC.Cli {
    public class PAMWriter : Object {
        public File file;

        public PAMWriter (File file) {
            this.file = file;
        }    

        public string get_conf_section () {
            string contents = read_contents (file);
            try {
                var regex = new Regex (Vars.PAM_CONF_REGEX);

                if (regex.match (contents)) {
                    int i = 0;
                    foreach (string str in regex.split (contents)) {
                        // Do not replace the contents of the PC plug section
                        if (i != 1) {
                            contents = contents.replace (str, "");
                        }

                        i++;
                    }
                } else {
                    return "";
                }
            } catch (RegexError e) {
                warning ("%s\n", e.message);
            }


            return contents;
        }

        public void remove_conf_section () {
            string contents = read_contents (file);
            string final_contents = contents.replace (get_conf_section (), "");

            try {
                FileUtils.set_contents (file.get_path (), final_contents);
            } catch (FileError e) {
                warning ("%s\n", e.message);
            }
        }

        public void remove_user_restrictions (string user) {
            string contents = read_contents (file);
            string conf_section = get_conf_section ();

            string new_conf = "";
            if (conf_section != "" && user != "") {
                int i = 0;
                string[] split = conf_section.split ("\n");
                foreach (string section_line in split) {
                    bool contains = (user in section_line);
                    if (!contains) {
                        if (i > 0) {
                            new_conf += "\n";
                        }

                        new_conf += section_line;
                    }

                    i++;
                }

                try {
                    FileUtils.set_contents (file.get_path (), contents.replace (conf_section, new_conf));
                } catch (FileError e) {
                    warning ("%s\n", e.message);
                }
            }            
        } 

        public void modify_user_restrictions (string user, bool enable) {
            string contents = read_contents (file);
            string conf_section = get_conf_section ();

            string new_conf = "";
            if (conf_section != "" && user != "") {
                int i = 0;
                string[] split = conf_section.split ("\n");
                foreach (string section_line in split) {
                    if (i > 0) {
                        new_conf += "\n";
                    }

                    bool contains = (user in section_line);
                    if (!contains) {
                        new_conf += section_line;
                    } else {
                        string prefix = "#";
                        if (enable) {
                            prefix = "";
                            section_line = section_line.replace ("#", "");
                        }

                        new_conf += prefix + section_line;
                    }

                    i++;
                }

                try {
                    FileUtils.set_contents (file.get_path (), contents.replace (conf_section, new_conf));
                } catch (FileError e) {
                    warning ("%s\n", e.message);
                }
            }            
        } 

        public void add_conf_line (string line, string? user = null) {
            string contents = read_contents (file);
            string conf_section = get_conf_section ();

            string new_conf = "";
            if (conf_section != "" && user != null) {
                int i = 0;
                string[] split = conf_section.split ("\n");
                foreach (string section_line in split) {
                    bool contains = (user in section_line);
                    if (!contains) {
                        if (i > 0) {
                            new_conf += "\n";
                        }

                        new_conf += section_line;
                    }

                    i++;
                }

                try {
                    FileUtils.set_contents (file.get_path (), contents.replace (conf_section, new_conf));
                } catch (FileError e) {
                    warning ("%s\n", e.message);
                }

                contents = read_contents (file);
                conf_section = get_conf_section ();
            }

            
            string final_contents = "";

            if (conf_section == "") {
                final_contents = contents +
                                    "\n" +
                                    Vars.PAM_CONF_START +
                                    "\n" +
                                    line +
                                    "\n" +
                                    Vars.PAM_CONF_END;
            } else {
                final_contents = conf_section.replace (Vars.PAM_CONF_END, "");
                final_contents += line + "\n" + Vars.PAM_CONF_END;
                final_contents = contents.replace (conf_section, final_contents);
            }

            try {
                FileUtils.set_contents (file.get_path (), final_contents);
            } catch (FileError e) {
                warning ("%s\n", e.message);
            }
        }  

        private string read_contents (File file) {
            string data = "";
            if (!file.query_exists ()) {
                return "";
            }

            try {
                FileUtils.get_contents (file.get_path (), out data);
            } catch (FileError e) {
                warning ("%s\n", e.message);
            }

            return data;
        }
    }
}